# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList


class PolicyRule(BaseModel):
    api_groups: Annotated[
        Optional[List[str]],
        Field(
            alias="apiGroups",
            description='APIGroups is the name of the APIGroup that contains the resources.  If multiple API groups are specified, any action requested against one of the enumerated resources in any API group will be allowed. "" represents the core API group and "*" represents all API groups.',
        ),
    ] = None
    non_resource_ur_ls: Annotated[
        Optional[List[str]],
        Field(
            alias="nonResourceURLs",
            description='NonResourceURLs is a set of partial urls that a user should have access to.  *s are allowed, but only as the full, final step in the path Since non-resource URLs are not namespaced, this field is only applicable for ClusterRoles referenced from a ClusterRoleBinding. Rules can either apply to API resources (such as "pods" or "secrets") or non-resource URL paths (such as "/api"),  but not both.',
        ),
    ] = None
    resource_names: Annotated[
        Optional[List[str]],
        Field(
            alias="resourceNames",
            description="ResourceNames is an optional white list of names that the rule applies to.  An empty set means that everything is allowed.",
        ),
    ] = None
    resources: Annotated[
        Optional[List[str]],
        Field(
            description="Resources is a list of resources this rule applies to. '*' represents all resources."
        ),
    ] = None
    verbs: Annotated[
        List[str],
        Field(
            description="Verbs is a list of Verbs that apply to ALL the ResourceKinds contained in this rule. '*' represents all verbs."
        ),
    ]


class RoleRef(BaseModel):
    api_group: Annotated[
        str,
        Field(
            alias="apiGroup",
            description="APIGroup is the group for the resource being referenced",
        ),
    ]
    kind: Annotated[str, Field(description="Kind is the type of resource being referenced")]
    name: Annotated[str, Field(description="Name is the name of resource being referenced")]


class Subject(BaseModel):
    api_group: Annotated[
        Optional[str],
        Field(
            alias="apiGroup",
            description='APIGroup holds the API group of the referenced subject. Defaults to "" for ServiceAccount subjects. Defaults to "rbac.authorization.k8s.io" for User and Group subjects.',
        ),
    ] = None
    kind: Annotated[
        str,
        Field(
            description='Kind of object being referenced. Values defined by this API group are "User", "Group", and "ServiceAccount". If the Authorizer does not recognized the kind value, the Authorizer should report an error.'
        ),
    ]
    name: Annotated[str, Field(description="Name of the object being referenced.")]
    namespace: Annotated[
        Optional[str],
        Field(
            description='Namespace of the referenced object.  If the object kind is non-namespace, such as "User" or "Group", and this value is not empty the Authorizer should report an error.'
        ),
    ] = None


class AggregationRule(BaseModel):
    cluster_role_selectors: Annotated[
        Optional[List[apimachinery.LabelSelector]],
        Field(
            alias="clusterRoleSelectors",
            description="ClusterRoleSelectors holds a list of selectors which will be used to find ClusterRoles and create the rules. If any of the selectors match, then the ClusterRole's permissions will be added",
        ),
    ] = None


class ClusterRole(Resource):
    aggregation_rule: Annotated[
        Optional[AggregationRule],
        Field(
            alias="aggregationRule",
            description="AggregationRule is an optional field that describes how to build the Rules for this ClusterRole. If AggregationRule is set, then the Rules are controller managed and direct changes to Rules will be stomped by the controller.",
        ),
    ] = None
    api_version: Annotated[
        Optional[Literal["rbac.authorization.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "rbac.authorization.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["ClusterRole"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "ClusterRole"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(description="Standard object's metadata."),
    ] = None
    rules: Annotated[
        Optional[List[PolicyRule]],
        Field(description="Rules holds all the PolicyRules for this ClusterRole"),
    ] = None


class ClusterRoleBinding(Resource):
    api_version: Annotated[
        Optional[Literal["rbac.authorization.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "rbac.authorization.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["ClusterRoleBinding"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "ClusterRoleBinding"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(description="Standard object's metadata."),
    ] = None
    role_ref: Annotated[
        RoleRef,
        Field(
            alias="roleRef",
            description="RoleRef can only reference a ClusterRole in the global namespace. If the RoleRef cannot be resolved, the Authorizer must return an error. This field is immutable.",
        ),
    ]
    subjects: Annotated[
        Optional[List[Subject]],
        Field(description="Subjects holds references to the objects the role applies to."),
    ] = None


ClusterRoleBindingList = ResourceList["ClusterRoleBinding"]


ClusterRoleList = ResourceList["ClusterRole"]


class Role(Resource):
    api_version: Annotated[
        Optional[Literal["rbac.authorization.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "rbac.authorization.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["Role"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "Role"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(description="Standard object's metadata."),
    ] = None
    rules: Annotated[
        Optional[List[PolicyRule]],
        Field(description="Rules holds all the PolicyRules for this Role"),
    ] = None


class RoleBinding(Resource):
    api_version: Annotated[
        Optional[Literal["rbac.authorization.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "rbac.authorization.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["RoleBinding"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "RoleBinding"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(description="Standard object's metadata."),
    ] = None
    role_ref: Annotated[
        RoleRef,
        Field(
            alias="roleRef",
            description="RoleRef can reference a Role in the current namespace or a ClusterRole in the global namespace. If the RoleRef cannot be resolved, the Authorizer must return an error. This field is immutable.",
        ),
    ]
    subjects: Annotated[
        Optional[List[Subject]],
        Field(description="Subjects holds references to the objects the role applies to."),
    ] = None


RoleBindingList = ResourceList["RoleBinding"]


RoleList = ResourceList["Role"]



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource

from . import v1


class SelfSubjectReviewStatus(BaseModel):
    user_info: Annotated[
        Optional[v1.UserInfo],
        Field(
            alias="userInfo",
            description="User attributes of the user making this request.",
        ),
    ] = None


class SelfSubjectReview(Resource):
    api_version: Annotated[
        Optional[Literal["authentication.k8s.io/v1beta1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "authentication.k8s.io/v1beta1"
    kind: Annotated[
        Optional[Literal["SelfSubjectReview"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "SelfSubjectReview"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    status: Annotated[
        Optional[SelfSubjectReviewStatus],
        Field(description="Status is filled in by the server with the user attributes."),
    ] = None



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, Dict, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList

from ..core import v1


class TokenRequest(BaseModel):
    audience: Annotated[
        str,
        Field(
            description='audience is the intended audience of the token in "TokenRequestSpec". It will default to the audiences of kube apiserver.'
        ),
    ]
    expiration_seconds: Annotated[
        Optional[int],
        Field(
            alias="expirationSeconds",
            description='expirationSeconds is the duration of validity of the token in "TokenRequestSpec". It has the same default value of "ExpirationSeconds" in "TokenRequestSpec".',
        ),
    ] = None


class VolumeNodeResources(BaseModel):
    count: Annotated[
        Optional[int],
        Field(
            description="count indicates the maximum number of unique volumes managed by the CSI driver that can be used on a node. A volume that is both attached and mounted on a node is considered to be used once, not twice. The same rule applies for a unique volume that is shared among multiple pods on the same node. If this field is not specified, then the supported number of volumes on this node is unbounded."
        ),
    ] = None


class CSIDriverSpec(BaseModel):
    attach_required: Annotated[
        Optional[bool],
        Field(
            alias="attachRequired",
            description="attachRequired indicates this CSI volume driver requires an attach operation (because it implements the CSI ControllerPublishVolume() method), and that the Kubernetes attach detach controller should call the attach volume interface which checks the volumeattachment status and waits until the volume is attached before proceeding to mounting. The CSI external-attacher coordinates with CSI volume driver and updates the volumeattachment status when the attach operation is complete. If the CSIDriverRegistry feature gate is enabled and the value is specified to false, the attach operation will be skipped. Otherwise the attach operation will be called.\n\nThis field is immutable.",
        ),
    ] = None
    fs_group_policy: Annotated[
        Optional[str],
        Field(
            alias="fsGroupPolicy",
            description="fsGroupPolicy defines if the underlying volume supports changing ownership and permission of the volume before being mounted. Refer to the specific FSGroupPolicy values for additional details.\n\nThis field was immutable in Kubernetes < 1.29 and now is mutable.\n\nDefaults to ReadWriteOnceWithFSType, which will examine each volume to determine if Kubernetes should modify ownership and permissions of the volume. With the default policy the defined fsGroup will only be applied if a fstype is defined and the volume's access mode contains ReadWriteOnce.",
        ),
    ] = None
    pod_info_on_mount: Annotated[
        Optional[bool],
        Field(
            alias="podInfoOnMount",
            description='podInfoOnMount indicates this CSI volume driver requires additional pod information (like podName, podUID, etc.) during mount operations, if set to true. If set to false, pod information will not be passed on mount. Default is false.\n\nThe CSI driver specifies podInfoOnMount as part of driver deployment. If true, Kubelet will pass pod information as VolumeContext in the CSI NodePublishVolume() calls. The CSI driver is responsible for parsing and validating the information passed in as VolumeContext.\n\nThe following VolumeContext will be passed if podInfoOnMount is set to true. This list might grow, but the prefix will be used. "csi.storage.k8s.io/pod.name": pod.Name "csi.storage.k8s.io/pod.namespace": pod.Namespace "csi.storage.k8s.io/pod.uid": string(pod.UID) "csi.storage.k8s.io/ephemeral": "true" if the volume is an ephemeral inline volume\n                                defined by a CSIVolumeSource, otherwise "false"\n\n"csi.storage.k8s.io/ephemeral" is a new feature in Kubernetes 1.16. It is only required for drivers which support both the "Persistent" and "Ephemeral" VolumeLifecycleMode. Other drivers can leave pod info disabled and/or ignore this field. As Kubernetes 1.15 doesn\'t support this field, drivers can only support one mode when deployed on such a cluster and the deployment determines which mode that is, for example via a command line parameter of the driver.\n\nThis field was immutable in Kubernetes < 1.29 and now is mutable.',
        ),
    ] = None
    requires_republish: Annotated[
        Optional[bool],
        Field(
            alias="requiresRepublish",
            description="requiresRepublish indicates the CSI driver wants `NodePublishVolume` being periodically called to reflect any possible change in the mounted volume. This field defaults to false.\n\nNote: After a successful initial NodePublishVolume call, subsequent calls to NodePublishVolume should only update the contents of the volume. New mount points will not be seen by a running container.",
        ),
    ] = None
    se_linux_mount: Annotated[
        Optional[bool],
        Field(
            alias="seLinuxMount",
            description='seLinuxMount specifies if the CSI driver supports "-o context" mount option.\n\nWhen "true", the CSI driver must ensure that all volumes provided by this CSI driver can be mounted separately with different `-o context` options. This is typical for storage backends that provide volumes as filesystems on block devices or as independent shared volumes. Kubernetes will call NodeStage / NodePublish with "-o context=xyz" mount option when mounting a ReadWriteOncePod volume used in Pod that has explicitly set SELinux context. In the future, it may be expanded to other volume AccessModes. In any case, Kubernetes will ensure that the volume is mounted only with a single SELinux context.\n\nWhen "false", Kubernetes won\'t pass any special SELinux mount options to the driver. This is typical for volumes that represent subdirectories of a bigger shared filesystem.\n\nDefault is "false".',
        ),
    ] = None
    storage_capacity: Annotated[
        Optional[bool],
        Field(
            alias="storageCapacity",
            description="storageCapacity indicates that the CSI volume driver wants pod scheduling to consider the storage capacity that the driver deployment will report by creating CSIStorageCapacity objects with capacity information, if set to true.\n\nThe check can be enabled immediately when deploying a driver. In that case, provisioning new volumes with late binding will pause until the driver deployment has published some suitable CSIStorageCapacity object.\n\nAlternatively, the driver can be deployed with the field unset or false and it can be flipped later when storage capacity information has been published.\n\nThis field was immutable in Kubernetes <= 1.22 and now is mutable.",
        ),
    ] = None
    token_requests: Annotated[
        Optional[List[TokenRequest]],
        Field(
            alias="tokenRequests",
            description='tokenRequests indicates the CSI driver needs pods\' service account tokens it is mounting volume for to do necessary authentication. Kubelet will pass the tokens in VolumeContext in the CSI NodePublishVolume calls. The CSI driver should parse and validate the following VolumeContext: "csi.storage.k8s.io/serviceAccount.tokens": {\n  "<audience>": {\n    "token": <token>,\n    "expirationTimestamp": <expiration timestamp in RFC3339>,\n  },\n  ...\n}\n\nNote: Audience in each TokenRequest should be different and at most one token is empty string. To receive a new token after expiry, RequiresRepublish can be used to trigger NodePublishVolume periodically.',
        ),
    ] = None
    volume_lifecycle_modes: Annotated[
        Optional[List[str]],
        Field(
            alias="volumeLifecycleModes",
            description='volumeLifecycleModes defines what kind of volumes this CSI volume driver supports. The default if the list is empty is "Persistent", which is the usage defined by the CSI specification and implemented in Kubernetes via the usual PV/PVC mechanism.\n\nThe other mode is "Ephemeral". In this mode, volumes are defined inline inside the pod spec with CSIVolumeSource and their lifecycle is tied to the lifecycle of that pod. A driver has to be aware of this because it is only going to get a NodePublishVolume call for such a volume.\n\nFor more information about implementing this mode, see https://kubernetes-csi.github.io/docs/ephemeral-local-volumes.html A driver can support one or more of these modes and more modes may be added in the future.\n\nThis field is beta. This field is immutable.',
        ),
    ] = None


class CSINodeDriver(BaseModel):
    allocatable: Annotated[
        Optional[VolumeNodeResources],
        Field(
            description="allocatable represents the volume resources of a node that are available for scheduling. This field is beta."
        ),
    ] = None
    name: Annotated[
        str,
        Field(
            description="name represents the name of the CSI driver that this object refers to. This MUST be the same name returned by the CSI GetPluginName() call for that driver."
        ),
    ]
    node_id: Annotated[
        str,
        Field(
            alias="nodeID",
            description='nodeID of the node from the driver point of view. This field enables Kubernetes to communicate with storage systems that do not share the same nomenclature for nodes. For example, Kubernetes may refer to a given node as "node1", but the storage system may refer to the same node as "nodeA". When Kubernetes issues a command to the storage system to attach a volume to a specific node, it can use this field to refer to the node name using the ID that the storage system will understand, e.g. "nodeA" instead of "node1". This field is required.',
        ),
    ]
    topology_keys: Annotated[
        Optional[List[str]],
        Field(
            alias="topologyKeys",
            description='topologyKeys is the list of keys supported by the driver. When a driver is initialized on a cluster, it provides a set of topology keys that it understands (e.g. "company.com/zone", "company.com/region"). When a driver is initialized on a node, it provides the same topology keys along with values. Kubelet will expose these topology keys as labels on its own node object. When Kubernetes does topology aware provisioning, it can use this list to determine which labels it should retrieve from the node object and pass back to the driver. It is possible for different nodes to use different topology keys. This can be empty if driver does not support topology.',
        ),
    ] = None


class CSINodeSpec(BaseModel):
    drivers: Annotated[
        List[CSINodeDriver],
        Field(
            description="drivers is a list of information of all CSI Drivers existing on a node. If all drivers in the list are uninstalled, this can become empty."
        ),
    ]


class VolumeError(BaseModel):
    message: Annotated[
        Optional[str],
        Field(
            description="message represents the error encountered during Attach or Detach operation. This string may be logged, so it should not contain sensitive information."
        ),
    ] = None
    time: Annotated[
        Optional[apimachinery.Time],
        Field(description="time represents the time the error was encountered."),
    ] = None


class CSIDriver(Resource):
    api_version: Annotated[
        Optional[Literal["storage.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "storage.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["CSIDriver"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "CSIDriver"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object metadata. metadata.Name indicates the name of the CSI driver that this object refers to; it MUST be the same name returned by the CSI GetPluginName() call for that driver. The driver name must be 63 characters or less, beginning and ending with an alphanumeric character ([a-z0-9A-Z]) with dashes (-), dots (.), and alphanumerics between. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        CSIDriverSpec,
        Field(description="spec represents the specification of the CSI Driver."),
    ]


CSIDriverList = ResourceList["CSIDriver"]


class CSINode(Resource):
    api_version: Annotated[
        Optional[Literal["storage.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "storage.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["CSINode"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "CSINode"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. metadata.name must be the Kubernetes node name."
        ),
    ] = None
    spec: Annotated[CSINodeSpec, Field(description="spec is the specification of CSINode")]


CSINodeList = ResourceList["CSINode"]


class CSIStorageCapacity(Resource):
    api_version: Annotated[
        Optional[Literal["storage.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "storage.k8s.io/v1"
    capacity: Annotated[
        Optional[apimachinery.Quantity],
        Field(
            description="capacity is the value reported by the CSI driver in its GetCapacityResponse for a GetCapacityRequest with topology and parameters that match the previous fields.\n\nThe semantic is currently (CSI spec 1.2) defined as: The available capacity, in bytes, of the storage that can be used to provision volumes. If not set, that information is currently unavailable."
        ),
    ] = None
    kind: Annotated[
        Optional[Literal["CSIStorageCapacity"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "CSIStorageCapacity"
    maximum_volume_size: Annotated[
        Optional[apimachinery.Quantity],
        Field(
            alias="maximumVolumeSize",
            description="maximumVolumeSize is the value reported by the CSI driver in its GetCapacityResponse for a GetCapacityRequest with topology and parameters that match the previous fields.\n\nThis is defined since CSI spec 1.4.0 as the largest size that may be used in a CreateVolumeRequest.capacity_range.required_bytes field to create a volume with the same parameters as those in GetCapacityRequest. The corresponding value in the Kubernetes API is ResourceRequirements.Requests in a volume claim.",
        ),
    ] = None
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. The name has no particular meaning. It must be a DNS subdomain (dots allowed, 253 characters). To ensure that there are no conflicts with other CSI drivers on the cluster, the recommendation is to use csisc-<uuid>, a generated name, or a reverse-domain name which ends with the unique CSI driver name.\n\nObjects are namespaced.\n\nMore info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    node_topology: Annotated[
        Optional[apimachinery.LabelSelector],
        Field(
            alias="nodeTopology",
            description="nodeTopology defines which nodes have access to the storage for which capacity was reported. If not set, the storage is not accessible from any node in the cluster. If empty, the storage is accessible from all nodes. This field is immutable.",
        ),
    ] = None
    storage_class_name: Annotated[
        str,
        Field(
            alias="storageClassName",
            description="storageClassName represents the name of the StorageClass that the reported capacity applies to. It must meet the same requirements as the name of a StorageClass object (non-empty, DNS subdomain). If that object no longer exists, the CSIStorageCapacity object is obsolete and should be removed by its creator. This field is immutable.",
        ),
    ]


CSIStorageCapacityList = ResourceList["CSIStorageCapacity"]


class StorageClass(Resource):
    allow_volume_expansion: Annotated[
        Optional[bool],
        Field(
            alias="allowVolumeExpansion",
            description="allowVolumeExpansion shows whether the storage class allow volume expand.",
        ),
    ] = None
    allowed_topologies: Annotated[
        Optional[List[v1.TopologySelectorTerm]],
        Field(
            alias="allowedTopologies",
            description="allowedTopologies restrict the node topologies where volumes can be dynamically provisioned. Each volume plugin defines its own supported topology specifications. An empty TopologySelectorTerm list means there is no topology restriction. This field is only honored by servers that enable the VolumeScheduling feature.",
        ),
    ] = None
    api_version: Annotated[
        Optional[Literal["storage.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "storage.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["StorageClass"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "StorageClass"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    mount_options: Annotated[
        Optional[List[str]],
        Field(
            alias="mountOptions",
            description='mountOptions controls the mountOptions for dynamically provisioned PersistentVolumes of this storage class. e.g. ["ro", "soft"]. Not validated - mount of the PVs will simply fail if one is invalid.',
        ),
    ] = None
    parameters: Annotated[
        Optional[Dict[str, str]],
        Field(
            description="parameters holds the parameters for the provisioner that should create volumes of this storage class."
        ),
    ] = None
    provisioner: Annotated[
        str, Field(description="provisioner indicates the type of the provisioner.")
    ]
    reclaim_policy: Annotated[
        Optional[str],
        Field(
            alias="reclaimPolicy",
            description="reclaimPolicy controls the reclaimPolicy for dynamically provisioned PersistentVolumes of this storage class. Defaults to Delete.",
        ),
    ] = None
    volume_binding_mode: Annotated[
        Optional[str],
        Field(
            alias="volumeBindingMode",
            description="volumeBindingMode indicates how PersistentVolumeClaims should be provisioned and bound.  When unset, VolumeBindingImmediate is used. This field is only honored by servers that enable the VolumeScheduling feature.",
        ),
    ] = None


StorageClassList = ResourceList["StorageClass"]


class VolumeAttachmentSource(BaseModel):
    inline_volume_spec: Annotated[
        Optional[v1.PersistentVolumeSpec],
        Field(
            alias="inlineVolumeSpec",
            description="inlineVolumeSpec contains all the information necessary to attach a persistent volume defined by a pod's inline VolumeSource. This field is populated only for the CSIMigration feature. It contains translated fields from a pod's inline VolumeSource to a PersistentVolumeSpec. This field is beta-level and is only honored by servers that enabled the CSIMigration feature.",
        ),
    ] = None
    persistent_volume_name: Annotated[
        Optional[str],
        Field(
            alias="persistentVolumeName",
            description="persistentVolumeName represents the name of the persistent volume to attach.",
        ),
    ] = None


class VolumeAttachmentSpec(BaseModel):
    attacher: Annotated[
        str,
        Field(
            description="attacher indicates the name of the volume driver that MUST handle this request. This is the name returned by GetPluginName()."
        ),
    ]
    node_name: Annotated[
        str,
        Field(
            alias="nodeName",
            description="nodeName represents the node that the volume should be attached to.",
        ),
    ]
    source: Annotated[
        VolumeAttachmentSource,
        Field(description="source represents the volume that should be attached."),
    ]


class VolumeAttachmentStatus(BaseModel):
    attach_error: Annotated[
        Optional[VolumeError],
        Field(
            alias="attachError",
            description="attachError represents the last error encountered during attach operation, if any. This field must only be set by the entity completing the attach operation, i.e. the external-attacher.",
        ),
    ] = None
    attached: Annotated[
        bool,
        Field(
            description="attached indicates the volume is successfully attached. This field must only be set by the entity completing the attach operation, i.e. the external-attacher."
        ),
    ]
    attachment_metadata: Annotated[
        Optional[Dict[str, str]],
        Field(
            alias="attachmentMetadata",
            description="attachmentMetadata is populated with any information returned by the attach operation, upon successful attach, that must be passed into subsequent WaitForAttach or Mount calls. This field must only be set by the entity completing the attach operation, i.e. the external-attacher.",
        ),
    ] = None
    detach_error: Annotated[
        Optional[VolumeError],
        Field(
            alias="detachError",
            description="detachError represents the last error encountered during detach operation, if any. This field must only be set by the entity completing the detach operation, i.e. the external-attacher.",
        ),
    ] = None


class VolumeAttachment(Resource):
    api_version: Annotated[
        Optional[Literal["storage.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "storage.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["VolumeAttachment"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "VolumeAttachment"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        VolumeAttachmentSpec,
        Field(
            description="spec represents specification of the desired attach/detach volume behavior. Populated by the Kubernetes system."
        ),
    ]
    status: Annotated[
        Optional[VolumeAttachmentStatus],
        Field(
            description="status represents status of the VolumeAttachment request. Populated by the entity completing the attach or detach operation, i.e. the external-attacher."
        ),
    ] = None


VolumeAttachmentList = ResourceList["VolumeAttachment"]



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList


class CrossVersionObjectReference(BaseModel):
    api_version: Annotated[
        Optional[str],
        Field(
            alias="apiVersion",
            description="apiVersion is the API version of the referent",
        ),
    ] = None
    kind: Annotated[
        str,
        Field(
            description="kind is the kind of the referent; More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ]
    name: Annotated[
        str,
        Field(
            description="name is the name of the referent; More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names"
        ),
    ]


class HPAScalingPolicy(BaseModel):
    period_seconds: Annotated[
        int,
        Field(
            alias="periodSeconds",
            description="periodSeconds specifies the window of time for which the policy should hold true. PeriodSeconds must be greater than zero and less than or equal to 1800 (30 min).",
        ),
    ]
    type: Annotated[str, Field(description="type is used to specify the scaling policy.")]
    value: Annotated[
        int,
        Field(
            description="value contains the amount of change which is permitted by the policy. It must be greater than zero"
        ),
    ]


class HPAScalingRules(BaseModel):
    policies: Annotated[
        Optional[List[HPAScalingPolicy]],
        Field(
            description="policies is a list of potential scaling polices which can be used during scaling. At least one policy must be specified, otherwise the HPAScalingRules will be discarded as invalid"
        ),
    ] = None
    select_policy: Annotated[
        Optional[str],
        Field(
            alias="selectPolicy",
            description="selectPolicy is used to specify which policy should be used. If not set, the default value Max is used.",
        ),
    ] = None
    stabilization_window_seconds: Annotated[
        Optional[int],
        Field(
            alias="stabilizationWindowSeconds",
            description="stabilizationWindowSeconds is the number of seconds for which past recommendations should be considered while scaling up or scaling down. StabilizationWindowSeconds must be greater than or equal to zero and less than or equal to 3600 (one hour). If not set, use the default values: - For scale up: 0 (i.e. no stabilization is done). - For scale down: 300 (i.e. the stabilization window is 300 seconds long).",
        ),
    ] = None


class HorizontalPodAutoscalerBehavior(BaseModel):
    scale_down: Annotated[
        Optional[HPAScalingRules],
        Field(
            alias="scaleDown",
            description="scaleDown is scaling policy for scaling Down. If not set, the default value is to allow to scale down to minReplicas pods, with a 300 second stabilization window (i.e., the highest recommendation for the last 300sec is used).",
        ),
    ] = None
    scale_up: Annotated[
        Optional[HPAScalingRules],
        Field(
            alias="scaleUp",
            description="scaleUp is scaling policy for scaling Up. If not set, the default value is the higher of:\n  * increase no more than 4 pods per 60 seconds\n  * double the number of pods per 60 seconds\nNo stabilization is used.",
        ),
    ] = None


class HorizontalPodAutoscalerCondition(BaseModel):
    last_transition_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastTransitionTime",
            description="lastTransitionTime is the last time the condition transitioned from one status to another",
        ),
    ] = None
    message: Annotated[
        Optional[str],
        Field(
            description="message is a human-readable explanation containing details about the transition"
        ),
    ] = None
    reason: Annotated[
        Optional[str],
        Field(description="reason is the reason for the condition's last transition."),
    ] = None
    status: Annotated[
        str,
        Field(description="status is the status of the condition (True, False, Unknown)"),
    ]
    type: Annotated[str, Field(description="type describes the current condition")]


class MetricTarget(BaseModel):
    average_utilization: Annotated[
        Optional[int],
        Field(
            alias="averageUtilization",
            description="averageUtilization is the target value of the average of the resource metric across all relevant pods, represented as a percentage of the requested value of the resource for the pods. Currently only valid for Resource metric source type",
        ),
    ] = None
    average_value: Annotated[
        Optional[apimachinery.Quantity],
        Field(
            alias="averageValue",
            description="averageValue is the target value of the average of the metric across all relevant pods (as a quantity)",
        ),
    ] = None
    type: Annotated[
        str,
        Field(
            description="type represents whether the metric type is Utilization, Value, or AverageValue"
        ),
    ]
    value: Annotated[
        Optional[apimachinery.Quantity],
        Field(description="value is the target value of the metric (as a quantity)."),
    ] = None


class MetricValueStatus(BaseModel):
    average_utilization: Annotated[
        Optional[int],
        Field(
            alias="averageUtilization",
            description="currentAverageUtilization is the current value of the average of the resource metric across all relevant pods, represented as a percentage of the requested value of the resource for the pods.",
        ),
    ] = None
    average_value: Annotated[
        Optional[apimachinery.Quantity],
        Field(
            alias="averageValue",
            description="averageValue is the current value of the average of the metric across all relevant pods (as a quantity)",
        ),
    ] = None
    value: Annotated[
        Optional[apimachinery.Quantity],
        Field(description="value is the current value of the metric (as a quantity)."),
    ] = None


class ResourceMetricSource(BaseModel):
    name: Annotated[str, Field(description="name is the name of the resource in question.")]
    target: Annotated[
        MetricTarget,
        Field(description="target specifies the target value for the given metric"),
    ]


class ResourceMetricStatus(BaseModel):
    current: Annotated[
        MetricValueStatus,
        Field(description="current contains the current value for the given metric"),
    ]
    name: Annotated[str, Field(description="name is the name of the resource in question.")]


class ContainerResourceMetricSource(BaseModel):
    container: Annotated[
        str,
        Field(
            description="container is the name of the container in the pods of the scaling target"
        ),
    ]
    name: Annotated[str, Field(description="name is the name of the resource in question.")]
    target: Annotated[
        MetricTarget,
        Field(description="target specifies the target value for the given metric"),
    ]


class ContainerResourceMetricStatus(BaseModel):
    container: Annotated[
        str,
        Field(
            description="container is the name of the container in the pods of the scaling target"
        ),
    ]
    current: Annotated[
        MetricValueStatus,
        Field(description="current contains the current value for the given metric"),
    ]
    name: Annotated[str, Field(description="name is the name of the resource in question.")]


class MetricIdentifier(BaseModel):
    name: Annotated[str, Field(description="name is the name of the given metric")]
    selector: Annotated[
        Optional[apimachinery.LabelSelector],
        Field(
            description="selector is the string-encoded form of a standard kubernetes label selector for the given metric When set, it is passed as an additional parameter to the metrics server for more specific metrics scoping. When unset, just the metricName will be used to gather metrics."
        ),
    ] = None


class ObjectMetricSource(BaseModel):
    described_object: Annotated[
        CrossVersionObjectReference,
        Field(
            alias="describedObject",
            description="describedObject specifies the descriptions of a object,such as kind,name apiVersion",
        ),
    ]
    metric: Annotated[
        MetricIdentifier,
        Field(description="metric identifies the target metric by name and selector"),
    ]
    target: Annotated[
        MetricTarget,
        Field(description="target specifies the target value for the given metric"),
    ]


class ObjectMetricStatus(BaseModel):
    current: Annotated[
        MetricValueStatus,
        Field(description="current contains the current value for the given metric"),
    ]
    described_object: Annotated[
        CrossVersionObjectReference,
        Field(
            alias="describedObject",
            description="DescribedObject specifies the descriptions of a object,such as kind,name apiVersion",
        ),
    ]
    metric: Annotated[
        MetricIdentifier,
        Field(description="metric identifies the target metric by name and selector"),
    ]


class PodsMetricSource(BaseModel):
    metric: Annotated[
        MetricIdentifier,
        Field(description="metric identifies the target metric by name and selector"),
    ]
    target: Annotated[
        MetricTarget,
        Field(description="target specifies the target value for the given metric"),
    ]


class PodsMetricStatus(BaseModel):
    current: Annotated[
        MetricValueStatus,
        Field(description="current contains the current value for the given metric"),
    ]
    metric: Annotated[
        MetricIdentifier,
        Field(description="metric identifies the target metric by name and selector"),
    ]


class ExternalMetricSource(BaseModel):
    metric: Annotated[
        MetricIdentifier,
        Field(description="metric identifies the target metric by name and selector"),
    ]
    target: Annotated[
        MetricTarget,
        Field(description="target specifies the target value for the given metric"),
    ]


class ExternalMetricStatus(BaseModel):
    current: Annotated[
        MetricValueStatus,
        Field(description="current contains the current value for the given metric"),
    ]
    metric: Annotated[
        MetricIdentifier,
        Field(description="metric identifies the target metric by name and selector"),
    ]


class MetricSpec(BaseModel):
    container_resource: Annotated[
        Optional[ContainerResourceMetricSource],
        Field(
            alias="containerResource",
            description='containerResource refers to a resource metric (such as those specified in requests and limits) known to Kubernetes describing a single container in each pod of the current scale target (e.g. CPU or memory). Such metrics are built in to Kubernetes, and have special scaling options on top of those available to normal per-pod metrics using the "pods" source. This is an alpha feature and can be enabled by the HPAContainerMetrics feature flag.',
        ),
    ] = None
    external: Annotated[
        Optional[ExternalMetricSource],
        Field(
            description="external refers to a global metric that is not associated with any Kubernetes object. It allows autoscaling based on information coming from components running outside of cluster (for example length of queue in cloud messaging service, or QPS from loadbalancer running outside of cluster)."
        ),
    ] = None
    object: Annotated[
        Optional[ObjectMetricSource],
        Field(
            description="object refers to a metric describing a single kubernetes object (for example, hits-per-second on an Ingress object)."
        ),
    ] = None
    pods: Annotated[
        Optional[PodsMetricSource],
        Field(
            description="pods refers to a metric describing each pod in the current scale target (for example, transactions-processed-per-second).  The values will be averaged together before being compared to the target value."
        ),
    ] = None
    resource: Annotated[
        Optional[ResourceMetricSource],
        Field(
            description='resource refers to a resource metric (such as those specified in requests and limits) known to Kubernetes describing each pod in the current scale target (e.g. CPU or memory). Such metrics are built in to Kubernetes, and have special scaling options on top of those available to normal per-pod metrics using the "pods" source.'
        ),
    ] = None
    type: Annotated[
        str,
        Field(
            description='type is the type of metric source.  It should be one of "ContainerResource", "External", "Object", "Pods" or "Resource", each mapping to a matching field in the object. Note: "ContainerResource" type is available on when the feature-gate HPAContainerMetrics is enabled'
        ),
    ]


class MetricStatus(BaseModel):
    container_resource: Annotated[
        Optional[ContainerResourceMetricStatus],
        Field(
            alias="containerResource",
            description='container resource refers to a resource metric (such as those specified in requests and limits) known to Kubernetes describing a single container in each pod in the current scale target (e.g. CPU or memory). Such metrics are built in to Kubernetes, and have special scaling options on top of those available to normal per-pod metrics using the "pods" source.',
        ),
    ] = None
    external: Annotated[
        Optional[ExternalMetricStatus],
        Field(
            description="external refers to a global metric that is not associated with any Kubernetes object. It allows autoscaling based on information coming from components running outside of cluster (for example length of queue in cloud messaging service, or QPS from loadbalancer running outside of cluster)."
        ),
    ] = None
    object: Annotated[
        Optional[ObjectMetricStatus],
        Field(
            description="object refers to a metric describing a single kubernetes object (for example, hits-per-second on an Ingress object)."
        ),
    ] = None
    pods: Annotated[
        Optional[PodsMetricStatus],
        Field(
            description="pods refers to a metric describing each pod in the current scale target (for example, transactions-processed-per-second).  The values will be averaged together before being compared to the target value."
        ),
    ] = None
    resource: Annotated[
        Optional[ResourceMetricStatus],
        Field(
            description='resource refers to a resource metric (such as those specified in requests and limits) known to Kubernetes describing each pod in the current scale target (e.g. CPU or memory). Such metrics are built in to Kubernetes, and have special scaling options on top of those available to normal per-pod metrics using the "pods" source.'
        ),
    ] = None
    type: Annotated[
        str,
        Field(
            description='type is the type of metric source.  It will be one of "ContainerResource", "External", "Object", "Pods" or "Resource", each corresponds to a matching field in the object. Note: "ContainerResource" type is available on when the feature-gate HPAContainerMetrics is enabled'
        ),
    ]


class HorizontalPodAutoscalerSpec(BaseModel):
    behavior: Annotated[
        Optional[HorizontalPodAutoscalerBehavior],
        Field(
            description="behavior configures the scaling behavior of the target in both Up and Down directions (scaleUp and scaleDown fields respectively). If not set, the default HPAScalingRules for scale up and scale down are used."
        ),
    ] = None
    max_replicas: Annotated[
        int,
        Field(
            alias="maxReplicas",
            description="maxReplicas is the upper limit for the number of replicas to which the autoscaler can scale up. It cannot be less that minReplicas.",
        ),
    ]
    metrics: Annotated[
        Optional[List[MetricSpec]],
        Field(
            description="metrics contains the specifications for which to use to calculate the desired replica count (the maximum replica count across all metrics will be used).  The desired replica count is calculated multiplying the ratio between the target value and the current value by the current number of pods.  Ergo, metrics used must decrease as the pod count is increased, and vice-versa.  See the individual metric source types for more information about how each type of metric must respond. If not set, the default metric will be set to 80% average CPU utilization."
        ),
    ] = None
    min_replicas: Annotated[
        Optional[int],
        Field(
            alias="minReplicas",
            description="minReplicas is the lower limit for the number of replicas to which the autoscaler can scale down.  It defaults to 1 pod.  minReplicas is allowed to be 0 if the alpha feature gate HPAScaleToZero is enabled and at least one Object or External metric is configured.  Scaling is active as long as at least one metric value is available.",
        ),
    ] = None
    scale_target_ref: Annotated[
        CrossVersionObjectReference,
        Field(
            alias="scaleTargetRef",
            description="scaleTargetRef points to the target resource to scale, and is used to the pods for which metrics should be collected, as well as to actually change the replica count.",
        ),
    ]


class HorizontalPodAutoscalerStatus(BaseModel):
    conditions: Annotated[
        Optional[List[HorizontalPodAutoscalerCondition]],
        Field(
            description="conditions is the set of conditions required for this autoscaler to scale its target, and indicates whether or not those conditions are met."
        ),
    ] = None
    current_metrics: Annotated[
        Optional[List[MetricStatus]],
        Field(
            alias="currentMetrics",
            description="currentMetrics is the last read state of the metrics used by this autoscaler.",
        ),
    ] = None
    current_replicas: Annotated[
        Optional[int],
        Field(
            alias="currentReplicas",
            description="currentReplicas is current number of replicas of pods managed by this autoscaler, as last seen by the autoscaler.",
        ),
    ] = None
    desired_replicas: Annotated[
        int,
        Field(
            alias="desiredReplicas",
            description="desiredReplicas is the desired number of replicas of pods managed by this autoscaler, as last calculated by the autoscaler.",
        ),
    ]
    last_scale_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastScaleTime",
            description="lastScaleTime is the last time the HorizontalPodAutoscaler scaled the number of pods, used by the autoscaler to control how often the number of pods is changed.",
        ),
    ] = None
    observed_generation: Annotated[
        Optional[int],
        Field(
            alias="observedGeneration",
            description="observedGeneration is the most recent generation observed by this autoscaler.",
        ),
    ] = None


class HorizontalPodAutoscaler(Resource):
    api_version: Annotated[
        Optional[Literal["autoscaling/v2"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "autoscaling/v2"
    kind: Annotated[
        Optional[Literal["HorizontalPodAutoscaler"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "HorizontalPodAutoscaler"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="metadata is the standard object metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[HorizontalPodAutoscalerSpec],
        Field(
            description="spec is the specification for the behaviour of the autoscaler. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status."
        ),
    ] = None
    status: Annotated[
        Optional[HorizontalPodAutoscalerStatus],
        Field(description="status is the current information about the autoscaler."),
    ] = None


HorizontalPodAutoscalerList = ResourceList["HorizontalPodAutoscaler"]



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList


class GroupVersionResource(BaseModel):
    group: Annotated[Optional[str], Field(description="The name of the group.")] = None
    resource: Annotated[Optional[str], Field(description="The name of the resource.")] = None
    version: Annotated[Optional[str], Field(description="The name of the version.")] = None


class StorageVersionMigrationSpec(BaseModel):
    continue_token: Annotated[
        Optional[str],
        Field(
            alias="continueToken",
            description='The token used in the list options to get the next chunk of objects to migrate. When the .status.conditions indicates the migration is "Running", users can use this token to check the progress of the migration.',
        ),
    ] = None
    resource: Annotated[
        GroupVersionResource,
        Field(
            description="The resource that is being migrated. The migrator sends requests to the endpoint serving the resource. Immutable."
        ),
    ]


class MigrationCondition(BaseModel):
    last_update_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastUpdateTime",
            description="The last time this condition was updated.",
        ),
    ] = None
    message: Annotated[
        Optional[str],
        Field(description="A human readable message indicating details about the transition."),
    ] = None
    reason: Annotated[
        Optional[str],
        Field(description="The reason for the condition's last transition."),
    ] = None
    status: Annotated[
        str, Field(description="Status of the condition, one of True, False, Unknown.")
    ]
    type: Annotated[str, Field(description="Type of the condition.")]


class StorageVersionMigrationStatus(BaseModel):
    conditions: Annotated[
        Optional[List[MigrationCondition]],
        Field(description="The latest available observations of the migration's current state."),
    ] = None
    resource_version: Annotated[
        Optional[str],
        Field(
            alias="resourceVersion",
            description="ResourceVersion to compare with the GC cache for performing the migration. This is the current resource version of given group, version and resource when kube-controller-manager first observes this StorageVersionMigration resource.",
        ),
    ] = None


class StorageVersionMigration(Resource):
    api_version: Annotated[
        Optional[Literal["storagemigration.k8s.io/v1alpha1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "storagemigration.k8s.io/v1alpha1"
    kind: Annotated[
        Optional[Literal["StorageVersionMigration"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "StorageVersionMigration"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[StorageVersionMigrationSpec],
        Field(description="Specification of the migration."),
    ] = None
    status: Annotated[
        Optional[StorageVersionMigrationStatus],
        Field(description="Status of the migration."),
    ] = None


StorageVersionMigrationList = ResourceList["StorageVersionMigration"]



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, Dict, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList


class CertificateSigningRequestSpec(BaseModel):
    expiration_seconds: Annotated[
        Optional[int],
        Field(
            alias="expirationSeconds",
            description="expirationSeconds is the requested duration of validity of the issued certificate. The certificate signer may issue a certificate with a different validity duration so a client must check the delta between the notBefore and and notAfter fields in the issued certificate to determine the actual duration.\n\nThe v1.22+ in-tree implementations of the well-known Kubernetes signers will honor this field as long as the requested duration is not greater than the maximum duration they will honor per the --cluster-signing-duration CLI flag to the Kubernetes controller manager.\n\nCertificate signers may not honor this field for various reasons:\n\n  1. Old signer that is unaware of the field (such as the in-tree\n     implementations prior to v1.22)\n  2. Signer whose configured maximum is shorter than the requested duration\n  3. Signer whose configured minimum is longer than the requested duration\n\nThe minimum valid value for expirationSeconds is 600, i.e. 10 minutes.",
        ),
    ] = None
    extra: Annotated[
        Optional[Dict[str, List[str]]],
        Field(
            description="extra contains extra attributes of the user that created the CertificateSigningRequest. Populated by the API server on creation and immutable."
        ),
    ] = None
    groups: Annotated[
        Optional[List[str]],
        Field(
            description="groups contains group membership of the user that created the CertificateSigningRequest. Populated by the API server on creation and immutable."
        ),
    ] = None
    request: Annotated[
        str,
        Field(
            description='request contains an x509 certificate signing request encoded in a "CERTIFICATE REQUEST" PEM block. When serialized as JSON or YAML, the data is additionally base64-encoded.'
        ),
    ]
    signer_name: Annotated[
        str,
        Field(
            alias="signerName",
            description='signerName indicates the requested signer, and is a qualified name.\n\nList/watch requests for CertificateSigningRequests can filter on this field using a "spec.signerName=NAME" fieldSelector.\n\nWell-known Kubernetes signers are:\n 1. "kubernetes.io/kube-apiserver-client": issues client certificates that can be used to authenticate to kube-apiserver.\n  Requests for this signer are never auto-approved by kube-controller-manager, can be issued by the "csrsigning" controller in kube-controller-manager.\n 2. "kubernetes.io/kube-apiserver-client-kubelet": issues client certificates that kubelets use to authenticate to kube-apiserver.\n  Requests for this signer can be auto-approved by the "csrapproving" controller in kube-controller-manager, and can be issued by the "csrsigning" controller in kube-controller-manager.\n 3. "kubernetes.io/kubelet-serving" issues serving certificates that kubelets use to serve TLS endpoints, which kube-apiserver can connect to securely.\n  Requests for this signer are never auto-approved by kube-controller-manager, and can be issued by the "csrsigning" controller in kube-controller-manager.\n\nMore details are available at https://k8s.io/docs/reference/access-authn-authz/certificate-signing-requests/#kubernetes-signers\n\nCustom signerNames can also be specified. The signer defines:\n 1. Trust distribution: how trust (CA bundles) are distributed.\n 2. Permitted subjects: and behavior when a disallowed subject is requested.\n 3. Required, permitted, or forbidden x509 extensions in the request (including whether subjectAltNames are allowed, which types, restrictions on allowed values) and behavior when a disallowed extension is requested.\n 4. Required, permitted, or forbidden key usages / extended key usages.\n 5. Expiration/certificate lifetime: whether it is fixed by the signer, configurable by the admin.\n 6. Whether or not requests for CA certificates are allowed.',
        ),
    ]
    uid: Annotated[
        Optional[str],
        Field(
            description="uid contains the uid of the user that created the CertificateSigningRequest. Populated by the API server on creation and immutable."
        ),
    ] = None
    usages: Annotated[
        Optional[List[str]],
        Field(
            description='usages specifies a set of key usages requested in the issued certificate.\n\nRequests for TLS client certificates typically request: "digital signature", "key encipherment", "client auth".\n\nRequests for TLS serving certificates typically request: "key encipherment", "digital signature", "server auth".\n\nValid values are:\n "signing", "digital signature", "content commitment",\n "key encipherment", "key agreement", "data encipherment",\n "cert sign", "crl sign", "encipher only", "decipher only", "any",\n "server auth", "client auth",\n "code signing", "email protection", "s/mime",\n "ipsec end system", "ipsec tunnel", "ipsec user",\n "timestamping", "ocsp signing", "microsoft sgc", "netscape sgc"'
        ),
    ] = None
    username: Annotated[
        Optional[str],
        Field(
            description="username contains the name of the user that created the CertificateSigningRequest. Populated by the API server on creation and immutable."
        ),
    ] = None


class CertificateSigningRequestCondition(BaseModel):
    last_transition_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastTransitionTime",
            description="lastTransitionTime is the time the condition last transitioned from one status to another. If unset, when a new condition type is added or an existing condition's status is changed, the server defaults this to the current time.",
        ),
    ] = None
    last_update_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastUpdateTime",
            description="lastUpdateTime is the time of the last update to this condition",
        ),
    ] = None
    message: Annotated[
        Optional[str],
        Field(
            description="message contains a human readable message with details about the request state"
        ),
    ] = None
    reason: Annotated[
        Optional[str],
        Field(description="reason indicates a brief reason for the request state"),
    ] = None
    status: Annotated[
        str,
        Field(
            description='status of the condition, one of True, False, Unknown. Approved, Denied, and Failed conditions may not be "False" or "Unknown".'
        ),
    ]
    type: Annotated[
        str,
        Field(
            description='type of the condition. Known conditions are "Approved", "Denied", and "Failed".\n\nAn "Approved" condition is added via the /approval subresource, indicating the request was approved and should be issued by the signer.\n\nA "Denied" condition is added via the /approval subresource, indicating the request was denied and should not be issued by the signer.\n\nA "Failed" condition is added via the /status subresource, indicating the signer failed to issue the certificate.\n\nApproved and Denied conditions are mutually exclusive. Approved, Denied, and Failed conditions cannot be removed once added.\n\nOnly one condition of a given type is allowed.'
        ),
    ]


class CertificateSigningRequestStatus(BaseModel):
    certificate: Annotated[
        Optional[str],
        Field(
            description='certificate is populated with an issued certificate by the signer after an Approved condition is present. This field is set via the /status subresource. Once populated, this field is immutable.\n\nIf the certificate signing request is denied, a condition of type "Denied" is added and this field remains empty. If the signer cannot issue the certificate, a condition of type "Failed" is added and this field remains empty.\n\nValidation requirements:\n 1. certificate must contain one or more PEM blocks.\n 2. All PEM blocks must have the "CERTIFICATE" label, contain no headers, and the encoded data\n  must be a BER-encoded ASN.1 Certificate structure as described in section 4 of RFC5280.\n 3. Non-PEM content may appear before or after the "CERTIFICATE" PEM blocks and is unvalidated,\n  to allow for explanatory text as described in section 5.2 of RFC7468.\n\nIf more than one PEM block is present, and the definition of the requested spec.signerName does not indicate otherwise, the first block is the issued certificate, and subsequent blocks should be treated as intermediate certificates and presented in TLS handshakes.\n\nThe certificate is encoded in PEM format.\n\nWhen serialized as JSON or YAML, the data is additionally base64-encoded, so it consists of:\n\n    base64(\n    -----BEGIN CERTIFICATE-----\n    ...\n    -----END CERTIFICATE-----\n    )'
        ),
    ] = None
    conditions: Annotated[
        Optional[List[CertificateSigningRequestCondition]],
        Field(
            description='conditions applied to the request. Known conditions are "Approved", "Denied", and "Failed".'
        ),
    ] = None


class CertificateSigningRequest(Resource):
    api_version: Annotated[
        Optional[Literal["certificates.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "certificates.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["CertificateSigningRequest"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "CertificateSigningRequest"
    metadata: Optional[apimachinery.ObjectMeta] = None
    spec: Annotated[
        CertificateSigningRequestSpec,
        Field(
            description="spec contains the certificate request, and is immutable after creation. Only the request, signerName, expirationSeconds, and usages fields can be set on creation. Other fields are derived by Kubernetes and cannot be modified by users."
        ),
    ]
    status: Annotated[
        Optional[CertificateSigningRequestStatus],
        Field(
            description="status contains information about whether the request is approved or denied, and the certificate issued by the signer, or the failure condition indicating signer failure."
        ),
    ] = None


CertificateSigningRequestList = ResourceList["CertificateSigningRequest"]



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList

from ..core import v1


class StatefulSetOrdinals(BaseModel):
    start: Annotated[
        Optional[int],
        Field(
            description="start is the number representing the first replica's index. It may be used to number replicas from an alternate index (eg: 1-indexed) over the default 0-indexed names, or to orchestrate progressive movement of replicas from one StatefulSet to another. If set, replica indices will be in the range:\n  [.spec.ordinals.start, .spec.ordinals.start + .spec.replicas).\nIf unset, defaults to 0. Replica indices will be in the range:\n  [0, .spec.replicas)."
        ),
    ] = None


class StatefulSetPersistentVolumeClaimRetentionPolicy(BaseModel):
    when_deleted: Annotated[
        Optional[str],
        Field(
            alias="whenDeleted",
            description="WhenDeleted specifies what happens to PVCs created from StatefulSet VolumeClaimTemplates when the StatefulSet is deleted. The default policy of `Retain` causes PVCs to not be affected by StatefulSet deletion. The `Delete` policy causes those PVCs to be deleted.",
        ),
    ] = None
    when_scaled: Annotated[
        Optional[str],
        Field(
            alias="whenScaled",
            description="WhenScaled specifies what happens to PVCs created from StatefulSet VolumeClaimTemplates when the StatefulSet is scaled down. The default policy of `Retain` causes PVCs to not be affected by a scaledown. The `Delete` policy causes the associated PVCs for any excess pods above the replica count to be deleted.",
        ),
    ] = None


class DaemonSetCondition(BaseModel):
    last_transition_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastTransitionTime",
            description="Last time the condition transitioned from one status to another.",
        ),
    ] = None
    message: Annotated[
        Optional[str],
        Field(description="A human readable message indicating details about the transition."),
    ] = None
    reason: Annotated[
        Optional[str],
        Field(description="The reason for the condition's last transition."),
    ] = None
    status: Annotated[
        str, Field(description="Status of the condition, one of True, False, Unknown.")
    ]
    type: Annotated[str, Field(description="Type of DaemonSet condition.")]


class DaemonSetStatus(BaseModel):
    collision_count: Annotated[
        Optional[int],
        Field(
            alias="collisionCount",
            description="Count of hash collisions for the DaemonSet. The DaemonSet controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ControllerRevision.",
        ),
    ] = None
    conditions: Annotated[
        Optional[List[DaemonSetCondition]],
        Field(
            description="Represents the latest available observations of a DaemonSet's current state."
        ),
    ] = None
    current_number_scheduled: Annotated[
        int,
        Field(
            alias="currentNumberScheduled",
            description="The number of nodes that are running at least 1 daemon pod and are supposed to run the daemon pod. More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
        ),
    ]
    desired_number_scheduled: Annotated[
        int,
        Field(
            alias="desiredNumberScheduled",
            description="The total number of nodes that should be running the daemon pod (including nodes correctly running the daemon pod). More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
        ),
    ]
    number_available: Annotated[
        Optional[int],
        Field(
            alias="numberAvailable",
            description="The number of nodes that should be running the daemon pod and have one or more of the daemon pod running and available (ready for at least spec.minReadySeconds)",
        ),
    ] = None
    number_misscheduled: Annotated[
        int,
        Field(
            alias="numberMisscheduled",
            description="The number of nodes that are running the daemon pod, but are not supposed to run the daemon pod. More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
        ),
    ]
    number_ready: Annotated[
        int,
        Field(
            alias="numberReady",
            description="numberReady is the number of nodes that should be running the daemon pod and have one or more of the daemon pod running with a Ready Condition.",
        ),
    ]
    number_unavailable: Annotated[
        Optional[int],
        Field(
            alias="numberUnavailable",
            description="The number of nodes that should be running the daemon pod and have none of the daemon pod running and available (ready for at least spec.minReadySeconds)",
        ),
    ] = None
    observed_generation: Annotated[
        Optional[int],
        Field(
            alias="observedGeneration",
            description="The most recent generation observed by the daemon set controller.",
        ),
    ] = None
    updated_number_scheduled: Annotated[
        Optional[int],
        Field(
            alias="updatedNumberScheduled",
            description="The total number of nodes that are running updated daemon pod",
        ),
    ] = None


class DeploymentCondition(BaseModel):
    last_transition_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastTransitionTime",
            description="Last time the condition transitioned from one status to another.",
        ),
    ] = None
    last_update_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastUpdateTime",
            description="The last time this condition was updated.",
        ),
    ] = None
    message: Annotated[
        Optional[str],
        Field(description="A human readable message indicating details about the transition."),
    ] = None
    reason: Annotated[
        Optional[str],
        Field(description="The reason for the condition's last transition."),
    ] = None
    status: Annotated[
        str, Field(description="Status of the condition, one of True, False, Unknown.")
    ]
    type: Annotated[str, Field(description="Type of deployment condition.")]


class DeploymentStatus(BaseModel):
    available_replicas: Annotated[
        Optional[int],
        Field(
            alias="availableReplicas",
            description="Total number of available pods (ready for at least minReadySeconds) targeted by this deployment.",
        ),
    ] = None
    collision_count: Annotated[
        Optional[int],
        Field(
            alias="collisionCount",
            description="Count of hash collisions for the Deployment. The Deployment controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ReplicaSet.",
        ),
    ] = None
    conditions: Annotated[
        Optional[List[DeploymentCondition]],
        Field(
            description="Represents the latest available observations of a deployment's current state."
        ),
    ] = None
    observed_generation: Annotated[
        Optional[int],
        Field(
            alias="observedGeneration",
            description="The generation observed by the deployment controller.",
        ),
    ] = None
    ready_replicas: Annotated[
        Optional[int],
        Field(
            alias="readyReplicas",
            description="readyReplicas is the number of pods targeted by this Deployment with a Ready Condition.",
        ),
    ] = None
    replicas: Annotated[
        Optional[int],
        Field(
            description="Total number of non-terminated pods targeted by this deployment (their labels match the selector)."
        ),
    ] = None
    unavailable_replicas: Annotated[
        Optional[int],
        Field(
            alias="unavailableReplicas",
            description="Total number of unavailable pods targeted by this deployment. This is the total number of pods that are still required for the deployment to have 100% available capacity. They may either be pods that are running but not yet available or pods that still have not been created.",
        ),
    ] = None
    updated_replicas: Annotated[
        Optional[int],
        Field(
            alias="updatedReplicas",
            description="Total number of non-terminated pods targeted by this deployment that have the desired template spec.",
        ),
    ] = None


class ReplicaSetCondition(BaseModel):
    last_transition_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastTransitionTime",
            description="The last time the condition transitioned from one status to another.",
        ),
    ] = None
    message: Annotated[
        Optional[str],
        Field(description="A human readable message indicating details about the transition."),
    ] = None
    reason: Annotated[
        Optional[str],
        Field(description="The reason for the condition's last transition."),
    ] = None
    status: Annotated[
        str, Field(description="Status of the condition, one of True, False, Unknown.")
    ]
    type: Annotated[str, Field(description="Type of replica set condition.")]


class ReplicaSetStatus(BaseModel):
    available_replicas: Annotated[
        Optional[int],
        Field(
            alias="availableReplicas",
            description="The number of available replicas (ready for at least minReadySeconds) for this replica set.",
        ),
    ] = None
    conditions: Annotated[
        Optional[List[ReplicaSetCondition]],
        Field(
            description="Represents the latest available observations of a replica set's current state."
        ),
    ] = None
    fully_labeled_replicas: Annotated[
        Optional[int],
        Field(
            alias="fullyLabeledReplicas",
            description="The number of pods that have labels matching the labels of the pod template of the replicaset.",
        ),
    ] = None
    observed_generation: Annotated[
        Optional[int],
        Field(
            alias="observedGeneration",
            description="ObservedGeneration reflects the generation of the most recently observed ReplicaSet.",
        ),
    ] = None
    ready_replicas: Annotated[
        Optional[int],
        Field(
            alias="readyReplicas",
            description="readyReplicas is the number of pods targeted by this ReplicaSet with a Ready Condition.",
        ),
    ] = None
    replicas: Annotated[
        int,
        Field(
            description="Replicas is the most recently observed number of replicas. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller/#what-is-a-replicationcontroller"
        ),
    ]


class RollingUpdateDaemonSet(BaseModel):
    max_surge: Annotated[
        Optional[apimachinery.IntOrString],
        Field(
            alias="maxSurge",
            description="The maximum number of nodes with an existing available DaemonSet pod that can have an updated DaemonSet pod during during an update. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). This can not be 0 if MaxUnavailable is 0. Absolute number is calculated from percentage by rounding up to a minimum of 1. Default value is 0. Example: when this is set to 30%, at most 30% of the total number of nodes that should be running the daemon pod (i.e. status.desiredNumberScheduled) can have their a new pod created before the old pod is marked as deleted. The update starts by launching new pods on 30% of nodes. Once an updated pod is available (Ready for at least minReadySeconds) the old DaemonSet pod on that node is marked deleted. If the old pod becomes unavailable for any reason (Ready transitions to false, is evicted, or is drained) an updated pod is immediatedly created on that node without considering surge limits. Allowing surge implies the possibility that the resources consumed by the daemonset on any given node can double if the readiness check fails, and so resource intensive daemonsets should take into account that they may cause evictions during disruption.",
        ),
    ] = None
    max_unavailable: Annotated[
        Optional[apimachinery.IntOrString],
        Field(
            alias="maxUnavailable",
            description="The maximum number of DaemonSet pods that can be unavailable during the update. Value can be an absolute number (ex: 5) or a percentage of total number of DaemonSet pods at the start of the update (ex: 10%). Absolute number is calculated from percentage by rounding up. This cannot be 0 if MaxSurge is 0 Default value is 1. Example: when this is set to 30%, at most 30% of the total number of nodes that should be running the daemon pod (i.e. status.desiredNumberScheduled) can have their pods stopped for an update at any given time. The update starts by stopping at most 30% of those DaemonSet pods and then brings up new DaemonSet pods in their place. Once the new pods are available, it then proceeds onto other DaemonSet pods, thus ensuring that at least 70% of original number of DaemonSet pods are available at all times during the update.",
        ),
    ] = None


class RollingUpdateDeployment(BaseModel):
    max_surge: Annotated[
        Optional[apimachinery.IntOrString],
        Field(
            alias="maxSurge",
            description="The maximum number of pods that can be scheduled above the desired number of pods. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). This can not be 0 if MaxUnavailable is 0. Absolute number is calculated from percentage by rounding up. Defaults to 25%. Example: when this is set to 30%, the new ReplicaSet can be scaled up immediately when the rolling update starts, such that the total number of old and new pods do not exceed 130% of desired pods. Once old pods have been killed, new ReplicaSet can be scaled up further, ensuring that total number of pods running at any time during the update is at most 130% of desired pods.",
        ),
    ] = None
    max_unavailable: Annotated[
        Optional[apimachinery.IntOrString],
        Field(
            alias="maxUnavailable",
            description="The maximum number of pods that can be unavailable during the update. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). Absolute number is calculated from percentage by rounding down. This can not be 0 if MaxSurge is 0. Defaults to 25%. Example: when this is set to 30%, the old ReplicaSet can be scaled down to 70% of desired pods immediately when the rolling update starts. Once new pods are ready, old ReplicaSet can be scaled down further, followed by scaling up the new ReplicaSet, ensuring that the total number of pods available at all times during the update is at least 70% of desired pods.",
        ),
    ] = None


class RollingUpdateStatefulSetStrategy(BaseModel):
    max_unavailable: Annotated[
        Optional[apimachinery.IntOrString],
        Field(
            alias="maxUnavailable",
            description="The maximum number of pods that can be unavailable during the update. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). Absolute number is calculated from percentage by rounding up. This can not be 0. Defaults to 1. This field is alpha-level and is only honored by servers that enable the MaxUnavailableStatefulSet feature. The field applies to all pods in the range 0 to Replicas-1. That means if there is any unavailable pod in the range 0 to Replicas-1, it will be counted towards MaxUnavailable.",
        ),
    ] = None
    partition: Annotated[
        Optional[int],
        Field(
            description="Partition indicates the ordinal at which the StatefulSet should be partitioned for updates. During a rolling update, all pods from ordinal Replicas-1 to Partition are updated. All pods from ordinal Partition-1 to 0 remain untouched. This is helpful in being able to do a canary based deployment. The default value is 0."
        ),
    ] = None


class StatefulSetCondition(BaseModel):
    last_transition_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastTransitionTime",
            description="Last time the condition transitioned from one status to another.",
        ),
    ] = None
    message: Annotated[
        Optional[str],
        Field(description="A human readable message indicating details about the transition."),
    ] = None
    reason: Annotated[
        Optional[str],
        Field(description="The reason for the condition's last transition."),
    ] = None
    status: Annotated[
        str, Field(description="Status of the condition, one of True, False, Unknown.")
    ]
    type: Annotated[str, Field(description="Type of statefulset condition.")]


class StatefulSetStatus(BaseModel):
    available_replicas: Annotated[
        Optional[int],
        Field(
            alias="availableReplicas",
            description="Total number of available pods (ready for at least minReadySeconds) targeted by this statefulset.",
        ),
    ] = None
    collision_count: Annotated[
        Optional[int],
        Field(
            alias="collisionCount",
            description="collisionCount is the count of hash collisions for the StatefulSet. The StatefulSet controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ControllerRevision.",
        ),
    ] = None
    conditions: Annotated[
        Optional[List[StatefulSetCondition]],
        Field(
            description="Represents the latest available observations of a statefulset's current state."
        ),
    ] = None
    current_replicas: Annotated[
        Optional[int],
        Field(
            alias="currentReplicas",
            description="currentReplicas is the number of Pods created by the StatefulSet controller from the StatefulSet version indicated by currentRevision.",
        ),
    ] = None
    current_revision: Annotated[
        Optional[str],
        Field(
            alias="currentRevision",
            description="currentRevision, if not empty, indicates the version of the StatefulSet used to generate Pods in the sequence [0,currentReplicas).",
        ),
    ] = None
    observed_generation: Annotated[
        Optional[int],
        Field(
            alias="observedGeneration",
            description="observedGeneration is the most recent generation observed for this StatefulSet. It corresponds to the StatefulSet's generation, which is updated on mutation by the API Server.",
        ),
    ] = None
    ready_replicas: Annotated[
        Optional[int],
        Field(
            alias="readyReplicas",
            description="readyReplicas is the number of pods created for this StatefulSet with a Ready Condition.",
        ),
    ] = None
    replicas: Annotated[
        int,
        Field(description="replicas is the number of Pods created by the StatefulSet controller."),
    ]
    update_revision: Annotated[
        Optional[str],
        Field(
            alias="updateRevision",
            description="updateRevision, if not empty, indicates the version of the StatefulSet used to generate Pods in the sequence [replicas-updatedReplicas,replicas)",
        ),
    ] = None
    updated_replicas: Annotated[
        Optional[int],
        Field(
            alias="updatedReplicas",
            description="updatedReplicas is the number of Pods created by the StatefulSet controller from the StatefulSet version indicated by updateRevision.",
        ),
    ] = None


class StatefulSetUpdateStrategy(BaseModel):
    rolling_update: Annotated[
        Optional[RollingUpdateStatefulSetStrategy],
        Field(
            alias="rollingUpdate",
            description="RollingUpdate is used to communicate parameters when Type is RollingUpdateStatefulSetStrategyType.",
        ),
    ] = None
    type: Annotated[
        Optional[str],
        Field(
            description="Type indicates the type of the StatefulSetUpdateStrategy. Default is RollingUpdate."
        ),
    ] = None


class ControllerRevision(Resource):
    api_version: Annotated[
        Optional[Literal["apps/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "apps/v1"
    data: Annotated[
        Optional[apimachinery.RawExtension],
        Field(description="Data is the serialized representation of the state."),
    ] = None
    kind: Annotated[
        Optional[Literal["ControllerRevision"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "ControllerRevision"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    revision: Annotated[
        int,
        Field(description="Revision indicates the revision of the state represented by Data."),
    ]


ControllerRevisionList = ResourceList["ControllerRevision"]


class DaemonSetUpdateStrategy(BaseModel):
    rolling_update: Annotated[
        Optional[RollingUpdateDaemonSet],
        Field(
            alias="rollingUpdate",
            description='Rolling update config params. Present only if type = "RollingUpdate".',
        ),
    ] = None
    type: Annotated[
        Optional[str],
        Field(
            description='Type of daemon set update. Can be "RollingUpdate" or "OnDelete". Default is RollingUpdate.'
        ),
    ] = None


class DeploymentStrategy(BaseModel):
    rolling_update: Annotated[
        Optional[RollingUpdateDeployment],
        Field(
            alias="rollingUpdate",
            description="Rolling update config params. Present only if DeploymentStrategyType = RollingUpdate.",
        ),
    ] = None
    type: Annotated[
        Optional[str],
        Field(
            description='Type of deployment. Can be "Recreate" or "RollingUpdate". Default is RollingUpdate.'
        ),
    ] = None


class DaemonSetSpec(BaseModel):
    min_ready_seconds: Annotated[
        Optional[int],
        Field(
            alias="minReadySeconds",
            description="The minimum number of seconds for which a newly created DaemonSet pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready).",
        ),
    ] = None
    revision_history_limit: Annotated[
        Optional[int],
        Field(
            alias="revisionHistoryLimit",
            description="The number of old history to retain to allow rollback. This is a pointer to distinguish between explicit zero and not specified. Defaults to 10.",
        ),
    ] = None
    selector: Annotated[
        apimachinery.LabelSelector,
        Field(
            description="A label query over pods that are managed by the daemon set. Must match in order to be controlled. It must match the pod template's labels. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors"
        ),
    ]
    template: Annotated[
        v1.PodTemplateSpec,
        Field(
            description='An object that describes the pod that will be created. The DaemonSet will create exactly one copy of this pod on every node that matches the template\'s node selector (or on every node if no node selector is specified). The only allowed template.spec.restartPolicy value is "Always". More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller#pod-template'
        ),
    ]
    update_strategy: Annotated[
        Optional[DaemonSetUpdateStrategy],
        Field(
            alias="updateStrategy",
            description="An update strategy to replace existing DaemonSet pods with new pods.",
        ),
    ] = None


class DeploymentSpec(BaseModel):
    min_ready_seconds: Annotated[
        Optional[int],
        Field(
            alias="minReadySeconds",
            description="Minimum number of seconds for which a newly created pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)",
        ),
    ] = None
    paused: Annotated[
        Optional[bool], Field(description="Indicates that the deployment is paused.")
    ] = None
    progress_deadline_seconds: Annotated[
        Optional[int],
        Field(
            alias="progressDeadlineSeconds",
            description="The maximum time in seconds for a deployment to make progress before it is considered to be failed. The deployment controller will continue to process failed deployments and a condition with a ProgressDeadlineExceeded reason will be surfaced in the deployment status. Note that progress will not be estimated during the time a deployment is paused. Defaults to 600s.",
        ),
    ] = None
    replicas: Annotated[
        Optional[int],
        Field(
            description="Number of desired pods. This is a pointer to distinguish between explicit zero and not specified. Defaults to 1."
        ),
    ] = None
    revision_history_limit: Annotated[
        Optional[int],
        Field(
            alias="revisionHistoryLimit",
            description="The number of old ReplicaSets to retain to allow rollback. This is a pointer to distinguish between explicit zero and not specified. Defaults to 10.",
        ),
    ] = None
    selector: Annotated[
        apimachinery.LabelSelector,
        Field(
            description="Label selector for pods. Existing ReplicaSets whose pods are selected by this will be the ones affected by this deployment. It must match the pod template's labels."
        ),
    ]
    strategy: Annotated[
        Optional[DeploymentStrategy],
        Field(description="The deployment strategy to use to replace existing pods with new ones."),
    ] = None
    template: Annotated[
        v1.PodTemplateSpec,
        Field(
            description='Template describes the pods that will be created. The only allowed template.spec.restartPolicy value is "Always".'
        ),
    ]


class ReplicaSetSpec(BaseModel):
    min_ready_seconds: Annotated[
        Optional[int],
        Field(
            alias="minReadySeconds",
            description="Minimum number of seconds for which a newly created pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)",
        ),
    ] = None
    replicas: Annotated[
        Optional[int],
        Field(
            description="Replicas is the number of desired replicas. This is a pointer to distinguish between explicit zero and unspecified. Defaults to 1. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller/#what-is-a-replicationcontroller"
        ),
    ] = None
    selector: Annotated[
        apimachinery.LabelSelector,
        Field(
            description="Selector is a label query over pods that should match the replica count. Label keys and values that must match in order to be controlled by this replica set. It must match the pod template's labels. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors"
        ),
    ]
    template: Annotated[
        Optional[v1.PodTemplateSpec],
        Field(
            description="Template is the object that describes the pod that will be created if insufficient replicas are detected. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller#pod-template"
        ),
    ] = None


class StatefulSetSpec(BaseModel):
    min_ready_seconds: Annotated[
        Optional[int],
        Field(
            alias="minReadySeconds",
            description="Minimum number of seconds for which a newly created pod should be ready without any of its container crashing for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)",
        ),
    ] = None
    ordinals: Annotated[
        Optional[StatefulSetOrdinals],
        Field(
            description='ordinals controls the numbering of replica indices in a StatefulSet. The default ordinals behavior assigns a "0" index to the first replica and increments the index by one for each additional replica requested.'
        ),
    ] = None
    persistent_volume_claim_retention_policy: Annotated[
        Optional[StatefulSetPersistentVolumeClaimRetentionPolicy],
        Field(
            alias="persistentVolumeClaimRetentionPolicy",
            description="persistentVolumeClaimRetentionPolicy describes the lifecycle of persistent volume claims created from volumeClaimTemplates. By default, all persistent volume claims are created as needed and retained until manually deleted. This policy allows the lifecycle to be altered, for example by deleting persistent volume claims when their stateful set is deleted, or when their pod is scaled down. This requires the StatefulSetAutoDeletePVC feature gate to be enabled, which is beta.",
        ),
    ] = None
    pod_management_policy: Annotated[
        Optional[str],
        Field(
            alias="podManagementPolicy",
            description="podManagementPolicy controls how pods are created during initial scale up, when replacing pods on nodes, or when scaling down. The default policy is `OrderedReady`, where pods are created in increasing order (pod-0, then pod-1, etc) and the controller will wait until each pod is ready before continuing. When scaling down, the pods are removed in the opposite order. The alternative policy is `Parallel` which will create pods in parallel to match the desired scale without waiting, and on scale down will delete all pods at once.",
        ),
    ] = None
    replicas: Annotated[
        Optional[int],
        Field(
            description="replicas is the desired number of replicas of the given Template. These are replicas in the sense that they are instantiations of the same Template, but individual replicas also have a consistent identity. If unspecified, defaults to 1."
        ),
    ] = None
    revision_history_limit: Annotated[
        Optional[int],
        Field(
            alias="revisionHistoryLimit",
            description="revisionHistoryLimit is the maximum number of revisions that will be maintained in the StatefulSet's revision history. The revision history consists of all revisions not represented by a currently applied StatefulSetSpec version. The default value is 10.",
        ),
    ] = None
    selector: Annotated[
        apimachinery.LabelSelector,
        Field(
            description="selector is a label query over pods that should match the replica count. It must match the pod template's labels. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors"
        ),
    ]
    service_name: Annotated[
        str,
        Field(
            alias="serviceName",
            description='serviceName is the name of the service that governs this StatefulSet. This service must exist before the StatefulSet, and is responsible for the network identity of the set. Pods get DNS/hostnames that follow the pattern: pod-specific-string.serviceName.default.svc.cluster.local where "pod-specific-string" is managed by the StatefulSet controller.',
        ),
    ]
    template: Annotated[
        v1.PodTemplateSpec,
        Field(
            description='template is the object that describes the pod that will be created if insufficient replicas are detected. Each pod stamped out by the StatefulSet will fulfill this Template, but have a unique identity from the rest of the StatefulSet. Each pod will be named with the format <statefulsetname>-<podindex>. For example, a pod in a StatefulSet named "web" with index number "3" would be named "web-3". The only allowed template.spec.restartPolicy value is "Always".'
        ),
    ]
    update_strategy: Annotated[
        Optional[StatefulSetUpdateStrategy],
        Field(
            alias="updateStrategy",
            description="updateStrategy indicates the StatefulSetUpdateStrategy that will be employed to update Pods in the StatefulSet when a revision is made to Template.",
        ),
    ] = None
    volume_claim_templates: Annotated[
        Optional[List[v1.PersistentVolumeClaim]],
        Field(
            alias="volumeClaimTemplates",
            description="volumeClaimTemplates is a list of claims that pods are allowed to reference. The StatefulSet controller is responsible for mapping network identities to claims in a way that maintains the identity of a pod. Every claim in this list must have at least one matching (by name) volumeMount in one container in the template. A claim in this list takes precedence over any volumes in the template, with the same name.",
        ),
    ] = None


class DaemonSet(Resource):
    api_version: Annotated[
        Optional[Literal["apps/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "apps/v1"
    kind: Annotated[
        Optional[Literal["DaemonSet"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "DaemonSet"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[DaemonSetSpec],
        Field(
            description="The desired behavior of this daemon set. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status"
        ),
    ] = None
    status: Annotated[
        Optional[DaemonSetStatus],
        Field(
            description="The current status of this daemon set. This data may be out of date by some window of time. Populated by the system. Read-only. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status"
        ),
    ] = None


DaemonSetList = ResourceList["DaemonSet"]


class Deployment(Resource):
    api_version: Annotated[
        Optional[Literal["apps/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "apps/v1"
    kind: Annotated[
        Optional[Literal["Deployment"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "Deployment"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[DeploymentSpec],
        Field(description="Specification of the desired behavior of the Deployment."),
    ] = None
    status: Annotated[
        Optional[DeploymentStatus],
        Field(description="Most recently observed status of the Deployment."),
    ] = None


DeploymentList = ResourceList["Deployment"]


class ReplicaSet(Resource):
    api_version: Annotated[
        Optional[Literal["apps/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "apps/v1"
    kind: Annotated[
        Optional[Literal["ReplicaSet"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "ReplicaSet"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="If the Labels of a ReplicaSet are empty, they are defaulted to be the same as the Pod(s) that the ReplicaSet manages. Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[ReplicaSetSpec],
        Field(
            description="Spec defines the specification of the desired behavior of the ReplicaSet. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status"
        ),
    ] = None
    status: Annotated[
        Optional[ReplicaSetStatus],
        Field(
            description="Status is the most recently observed status of the ReplicaSet. This data may be out of date by some window of time. Populated by the system. Read-only. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status"
        ),
    ] = None


ReplicaSetList = ResourceList["ReplicaSet"]


class StatefulSet(Resource):
    api_version: Annotated[
        Optional[Literal["apps/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "apps/v1"
    kind: Annotated[
        Optional[Literal["StatefulSet"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "StatefulSet"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[StatefulSetSpec],
        Field(description="Spec defines the desired identities of pods in this set."),
    ] = None
    status: Annotated[
        Optional[StatefulSetStatus],
        Field(
            description="Status is the current status of Pods in this StatefulSet. This data may be out of date by some window of time."
        ),
    ] = None


StatefulSetList = ResourceList["StatefulSet"]



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList


class ParentReference(BaseModel):
    group: Annotated[
        Optional[str],
        Field(description="Group is the group of the object being referenced."),
    ] = None
    name: Annotated[str, Field(description="Name is the name of the object being referenced.")]
    namespace: Annotated[
        Optional[str],
        Field(description="Namespace is the namespace of the object being referenced."),
    ] = None
    resource: Annotated[
        str,
        Field(description="Resource is the resource of the object being referenced."),
    ]


class ServiceCIDRSpec(BaseModel):
    cidrs: Annotated[
        Optional[List[str]],
        Field(
            description='CIDRs defines the IP blocks in CIDR notation (e.g. "192.168.0.0/24" or "2001:db8::/64") from which to assign service cluster IPs. Max of two CIDRs is allowed, one of each IP family. This field is immutable.'
        ),
    ] = None


class IPAddressSpec(BaseModel):
    parent_ref: Annotated[
        ParentReference,
        Field(
            alias="parentRef",
            description="ParentRef references the resource that an IPAddress is attached to. An IPAddress must reference a parent object.",
        ),
    ]


class IPAddress(Resource):
    api_version: Annotated[
        Optional[Literal["networking.k8s.io/v1beta1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "networking.k8s.io/v1beta1"
    kind: Annotated[
        Optional[Literal["IPAddress"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "IPAddress"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[IPAddressSpec],
        Field(
            description="spec is the desired state of the IPAddress. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status"
        ),
    ] = None


IPAddressList = ResourceList["IPAddress"]


class ServiceCIDRStatus(BaseModel):
    conditions: Annotated[
        Optional[List[apimachinery.Condition]],
        Field(
            description="conditions holds an array of metav1.Condition that describe the state of the ServiceCIDR. Current service state"
        ),
    ] = None


class ServiceCIDR(Resource):
    api_version: Annotated[
        Optional[Literal["networking.k8s.io/v1beta1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "networking.k8s.io/v1beta1"
    kind: Annotated[
        Optional[Literal["ServiceCIDR"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "ServiceCIDR"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[ServiceCIDRSpec],
        Field(
            description="spec is the desired state of the ServiceCIDR. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status"
        ),
    ] = None
    status: Annotated[
        Optional[ServiceCIDRStatus],
        Field(
            description="status represents the current state of the ServiceCIDR. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status"
        ),
    ] = None


ServiceCIDRList = ResourceList["ServiceCIDR"]
